import json
from urllib.parse import urljoin
import requests
from .exception import *
from .utils import trigger
from .config import Configure


class Base(object):
    def __init__(self, key):
        self.baseURL = Configure().get(key)

    def _check(self, *args, **kwargs):
        if not self.baseURL:
            raise RuntimeError('baseURL must init and supplied')

    @trigger(before=_check)
    def create(self, *args):
        if len(args) == 1:
            dst, = args
            res = requests.post(self.baseURL, json.dumps({'dst': dst}))
        elif len(args) == 2:
            src, dst = args
            res = requests.post(self.baseURL, json.dumps({'src': src, 'dst': dst}))
        else:
            raise RuntimeError('args too many.')

        if res.status_code == 200:
            rst = res.json()
            assert 'src' in rst
            return rst['src']
        else:
            content = res.content.decode('utf-8')
            raise code2exception_mapper[res.status_code](content) \
                if res.status_code in code2exception_mapper else UnknownError(content)

    @trigger(before=_check)
    def update(self, src: str, dst: str):
        res = requests.put(urljoin(self.baseURL + '/', src), json.dumps({'dst': dst}))
        if res.status_code != 200:
            content = res.content.decode('utf-8')
            raise code2exception_mapper[res.status_code](content) \
                if res.status_code in code2exception_mapper else UnknownError(content)

    @trigger(before=_check)
    def retrieve(self, src):
        res = requests.get(urljoin(self.baseURL + '/', src), allow_redirects=False)
        if res.status_code not in (301, 302):
            content = res.content.decode('utf-8')
            raise code2exception_mapper[res.status_code](content) \
                if res.status_code in code2exception_mapper else UnknownError(content)

        return res.headers['Location']

    @trigger(before=_check)
    def delete(self, src):
        res = requests.delete(urljoin(self.baseURL + '/', src))
        if res.status_code != 200:
            content = res.content.decode('utf-8')
            raise code2exception_mapper[res.status_code](content) \
                if res.status_code in code2exception_mapper else UnknownError(content)



"""
some utilities, decorators in this file.
"""
import functools


class SingletonDecorator(object):
    """
    class decorator, wrap class to singleton
    """

    def __init__(self, cls):
        self.cls = cls
        self.instance = None

    def __call__(self, *args, **kwargs):
        if not self.instance:
            self.instance = self.cls(*args, **kwargs)
        return self.instance


def trigger(before=None, after=None):
    def wrapper(func):
        @functools.wraps(func)
        def inner_wrapper(*args, **kwargs):
            before(*args, **kwargs) if before and callable(before) else None
            rst = func(*args, **kwargs)
            after(*args, **kwargs) if after and callable(after) else None
            return rst

        return inner_wrapper

    return wrapper



"""
some user defined exception here.
"""


class CustomException(Exception):
    pass


class ArgumentsError(CustomException):
    # response status code 400
    pass


class ServerError(CustomException):
    # response status code 500
    pass


class NotFoundError(CustomException):
    # response status code 404
    pass


class UnknownError(CustomException):
    pass


code2exception_mapper = {
    400: ArgumentsError,
    404: NotFoundError,
    500: ServerError,
}


from setuptools import setup

setup(
    name='portalutils',
    packages=['portalutils'],
    version='0.0.2',
    url='https://github.com/Jack-Kingdom/portalutils.git',
    description='Portal access utility.',
    long_description=open('README.md').read(),

    author='Jack King',
    author_email='email@qiaohong.org',
    license='MIT',

    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Natural Language :: English',
    ], install_requires=['requests']
)



#! bin/python
"""
configure store
"""

from .utils import SingletonDecorator


@SingletonDecorator
class Configure(object):

    def __init__(self):
        self.store = {
            'alias': 'http://localhost:8888/api/v1/alias',
            'shortURL': 'http://localhost:8888/api/v1/shortURL',
        }

    def register(self, key: str, value: str):
        if key not in self.store:
            raise IndexError('key not in configure')

        self.store[key] = value

    def get(self, key: str):
        if key not in self.store:
            raise IndexError('key not in configure')

        return self.store[key]
